/**
 * SortedArray is a sorted mutable collection.
 *
 * SortedArray values are maintained in sorted order.
 *
 * SortedArray values must have a total ordering.
 * The total ordering of values must not change while they are stored in the SortedArray.
 *
 * Methods for adding values:
 *
 * - {@link SortedArray#add}
 * - {@link SortedArray#update}
 *
 * Methods for removing values:
 *
 * - {@link SortedArray#clear}
 * - {@link SortedArray#delete}
 * - {@link SortedArray#deleteAt}
 * - {@link SortedArray#deleteSlice}
 * - {@link SortedArray#pop}
 *
 * Methods for looking up values:
 *
 * - {@link SortedArray#at}
 * - {@link SortedArray#bisectLeft}
 * - {@link SortedArray#bisectRight}
 * - {@link SortedArray#count}
 * - {@link SortedArray#includes}
 * - {@link SortedArray#indexOf}
 *
 * Methods for iterating values:
 *
 * - {@link SortedArray#irange}
 * - {@link SortedArray#islice}
 * - {@link SortedArray#slice}
 * - {@link SortedArray#[Symbol.iterator]}
 *
 * Miscellaneous methods and properties:
 *
 * - {@link SortedArray#clone}
 * - {@link SortedArray#concat}
 * - {@link SortedArray#length}
 * - {@link SortedArray#toJSON}
 * - {@link SortedArray#toString}
 *
 * @typeParam T - The element type.
 */
declare class SortedArray<T> {
    static readonly DEFAULT_LOAD_FACTOR = 1000;
    private readonly _load;
    private readonly _cmp;
    private _len;
    private _lists;
    private _maxes;
    private _index;
    private _offset;
    /**
     * Initialize SortedArray instance.
     *
     * Optional `iterable` argument provides an initial iterable of values to initialize the SortedArray.
     *
     * @param iterable - Initial values (optional).
     *
     * @param {object} options - An object that specifies characteristics about the sorted container.
     * @param options.comparator - Function used to determine the order of the elements.
     * It should return a negative number if the first argument is less than the second argument,
     * a positive number if the first argument is greater than the second argument, or a zero otherwise.
     * The comparator should always return a number, and never return NaN.
     * In addition, the comparator has the same requirements as the parameter of `Array.prototype.sort`
     * (purity, stability, reflexivity, anti-symmetry, and transitivity).
     *
     * If omitted, the elements are compared with `<` and `>` operators.
     * These operators violate transitivity if the inputs have mixed types or include NaN.
     * 【TODO: add link to docs explaining】
     *
     * @param options.loadFactor - Specifies the load-factor for SortedArray sublists.
     * The default load factor of 1000 works well for lists from tens to tens-of-millions of values.
     * Good practice is to use a value that is the cube root of the list size.
     * With billions of elements, the best load factor depends on your usage.
     * It's best to leave the load factor at the default until you start benchmarking.
     *
     * @see https://grantjenks.com/docs/sortedcontainers/implementation.html
     * @see https://grantjenks.com/docs/sortedcontainers/performance-scale.html
     */
    constructor(iterable?: Iterable<T>, options?: {
        comparator?: (a: T, b: T) => number;
        loadFactor?: number;
    });
    /**
     * Remove all values from SortedArray.
     */
    clear(): void;
    /**
     * Add `value` to SortedArray.
     *
     * @example
     * const sl = new SortedArray();
     * sl.add(3);
     * sl.add(1);
     * sl.add(2);
     * sl // SortedArray [1, 2, 3]
     *
     * @param value - Value to add to SortedArray.
     */
    add(value: T): void;
    /**
     * Split sublists with length greater than double the load-factor.
     *
     * Updates the index when the sublist length is less than double the load level.
     * This requires incrementing the nodes in a traversal from the leaf node to the root.
     * For an example traversal see {@link _loc}.
     */
    private _expand;
    /**
     * Update SortedArray by adding all values from `iterable`.
     *
     * @example
     * const sl = new SortedArray();
     * sl.update([3, 1, 2]);
     * sl // SortedArray [1, 2, 3]
     *
     * @param iterable - Iterable of values to add.
     */
    update(iterable: Iterable<T>): void;
    /**
     * Return true if `value` is an element of the SortedArray.
     *
     * @example
     * const sl = new SortedArray([1, 2, 3, 4, 5]);
     * sl.includes(3) // true
     *
     * @param value - Search for value in SortedArray.
     * @returns True if `value` in SortedArray.
     */
    includes(value: T): boolean;
    /**
     * Remove one `value` from SortedArray if it is a member.
     *
     * If `value` is not a member, do nothing.
     *
     * @example
     * const sl = new SortedArray([1, 2, 3, 4, 5])
     * sl.delete(5)
     * sl.delete(0)
     * Array.from(sl) // [1, 2, 3, 4]
     * @param value - Value to discard from SortedArray.
     * @returns Returns true if an element in the SortedArray existed and has been removed, or false if the element does not exist.
     */
    delete(value: T): boolean;
    /**
     * Delete value at the given `(pos, idx)`.
     *
     * Combines lists that are less than half the load level.
     *
     * Updates the index when the sublist length is more than half the load level.
     * This requires decrementing the nodes in a traversal from the leaf node to the root.
     * For an example traversal see {@link _loc}.
     *
     * @param pos - Lists index.
     * @param idx - Sublist index.
     */
    private _delete;
    /**
     * Convert an index pair (lists index, sublist index) into a single index number that corresponds to the position of the value in the SortedArray.
     *
     * Many queries require the index be built.
     * Details of the index are described in {@link _build_index}.
     *
     * Indexing requires traversing the tree from a leaf node to the root.
     * The parent of each node is easily computable at `(pos - 1) >>> 1`.
     *
     * Left-child nodes are always at odd indices and right-child nodes are always at even indices.
     *
     * When traversing up from a right-child node, increment the total by the left-child node.
     *
     * The final index is the sum from traversal and the index in the sublist.
     *
     * @param pos - Lists index.
     * @param idx - Sublist index.
     * @returns Index in SortedArray.
     */
    private _loc;
    /**
     * Convert an index into an index pair (lists index, sublist index) that can be used to access the corresponding lists position.
     *
     * Many queries require the index be built.
     * Details of the index are described in {@link _build_index}.
     *
     * Indexing requires traversing the tree to a leaf node.
     * Each node has two children which are easily computable.
     * Given an index, pos, the left-child is at `pos * 2 + 1` and the right-child is at `pos * 2 + 2`.
     *
     * When the index is less than the left-child, traversal moves to the left sub-tree.
     * Otherwise, the index is decremented by the left-child and traversal moves to the right sub-tree.
     *
     * At a child node, the indexing pair is computed from the relative position of the child node as compared with the offset and the remaining index.
     *
     * @param idx - Index in SortedArray.
     * @returns (lists index, sublist index) pair.
     */
    private _pos;
    /**
     * Build a positional index for indexing the SortedArray.
     *
     * Indexes are represented as binary trees in a dense array notation similar to a binary heap.
     *
     * When built, the index can be used for efficient indexing into the list.
     * See the comment and notes on {@link _pos} for details.
     */
    private _buildIndex;
    /**
     * Remove value at `index` from SortedArray.
     *
     * If `index` is out of range, does nothing.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.deleteAt(2);
     * sl // SortedArray ['a', 'b', 'd', 'e']
     *
     * @param index - Integer or slice for indexing. Negative integers count back from the last item in the array.
     */
    deleteAt(index: number): void;
    /**
     * Removes elements from SortedArray by index.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.deleteSlice(0, 2);
     * sl // SortedArray ['c', 'd', 'e']
     *
     * @param start The zero-based location in the array from which to start removing elements.
     * Negative integers count back from the last item in the array.
     * @param end The zero-based location in the array at which to stop removing elements.
     * Negative integers count back from the last item in the array.
     * Elements up to but not including `end` are removed.
     */
    deleteSlice(start?: number, end?: number): void;
    /**
     * Returns a copy of a section of SortedArray as an ordinary Array.
     *
     * For both start and end, a negative index can be used to indicate an offset from the end of the array.
     *
     * @param start The beginning index of the specified portion of the array.
     * If start is undefined, then the slice begins at index 0.
     * @param end The end index of the specified portion of the array. This is exclusive of the element at the index `end`.
     * If end is undefined, then the slice extends to the end of the array.
     */
    slice(start?: number, end?: number): T[];
    /**
     * Lookup value at `index` in SortedArray.
     *
     * If `index` is out of range, returns `undefined`.
     *
     * @example
     * const sl = new SortedList('abcde');
     * sl.at(1) // 'b'
     * sl.at(-1) // 'e'
     * sl.at(7) // undefined
     *
     * @param index - The zero-based index of the desired code unit. A negative index will count back from the last item.
     * @returns The item located at the specified index, or `undefined` if index out of range.
     */
    at(index: number): T | undefined;
    /**
     * Return an iterator over the SortedArray.
     *
     * Iterating a SortedArray while adding or deleting elements may throw an error or silently fail to iterate over all elements.
     */
    [Symbol.iterator](): IterableIterator<T>;
    /**
     * Return a reverse iterator over the SortedArray.
     *
     * Iterating a SortedArray while adding or deleting elements may throw an error or silently fail to iterate over all elements.
     */
    reversed(): IterableIterator<T>;
    /**
     * Return an iterator that slices SortedArray from `start` to `stop`.
     *
     * The `start` and `stop` index are treated inclusive and exclusive, respectively.
     *
     * A negative index will count back from the last item.
     *
     * Both `start` and `stop` default to `undefined` which is automatically inclusive of the beginning and end of the SortedArray.
     *
     * When `reverse` is `true` the values are yielded from the iterator in reverse order; `reverse` defaults to `false`.
     *
     * Iterating a SortedArray while adding or deleting elements may throw an error or silently fail to iterate over all elements.
     *
     * @example
     * const sl = new SortedArray('abcdefghij');
     * const it = sl.islice(2, 6);
     * Array.from(it) // ['c', 'd', 'e', 'f']
     *
     * @param start - Start index (inclusive).
     * @param end - Stop index (exclusive).
     * @param reverse - Yield values in reverse order.
     * @returns Iterator.
     */
    islice(start?: number, end?: number, reverse?: boolean): IterableIterator<T>;
    /**
     * Return an iterator that slices SortedArray using two index pairs.
     *
     * The index pairs are (min_pos, min_idx) and (max_pos, max_idx), the first inclusive and the latter exclusive.
     * See {@link _pos} for details on how an index is converted to an index pair.
     *
     * When `reverse` is `true`, values are yielded from the iterator in reverse order.
     */
    private _islice;
    /**
     * Create an iterator of values between `minimum` and `maximum`.
     *
     * Both `minimum` and `maximum` default to `undefined` which is automatically inclusive of the beginning and end of the SortedArray.
     *
     * The argument `includeMinimum` and `includeMaximum` is a pair of booleans that indicates whether the minimum and maximum ought to be included in the range, respectively.
     * Both arguments default to `true` such that the range is inclusive of both minimum and maximum.
     *
     * When `reverse` is `true` the values are yielded from the iterator in reverse order; `reverse` defaults to `false`.
     *
     * Iterating a SortedArray while adding or deleting elements may throw an error or silently fail to iterate over all elements.
     *
     * @example
     * const sl = new SortedArray('abcdefghij');
     * const it = sl.irange('c', 'f');
     * Array.from(it) // ['c', 'd', 'e', 'f']
     *
     * @param minimum - Minimum value to start iterating.
     * @param maximum - Maximum value to stop iterating.
     * @param includeMinimum - Whether the minimum ought to be included in the range.
     * @param includeMaximum - Whether the maximum ought to be included in the range.
     * @param reverse - Yield values in reverse order.
     * @returns Iterator.
     */
    irange(minimum?: T, maximum?: T, includeMinimum?: boolean, includeMaximum?: boolean, reverse?: boolean): IterableIterator<T>;
    /**
     * The size of the SortedArray.
     */
    get length(): number;
    /**
     * Return an index to insert `value` in the SortedArray.
     *
     * If the `value` is already present, the insertion point will be before (to the left of) any existing values.
     *
     * @example
     * const sl = new SortedArray([10, 11, 12, 13, 14]);
     * sl.bisect_left(12) // 2
     *
     * @param value - Insertion index of value in SortedArray.
     * @returns Index.
     */
    bisectLeft(value: T): number;
    /**
     * Return an index to insert `value` in the SortedArray.
     *
     * Similar to {@link bisectLeft}, but if `value` is already present, the insertion point will be after (to the right of) any existing values.
     *
     * @example
     * const sl = new SortedArray([10, 11, 12, 13, 14]);
     * sl.bisect_right(12) // 3
     *
     * @param value - Insertion index of value in SortedArray.
     * @returns Index.
     */
    bisectRight(value: T): number;
    /**
     * Return number of occurrences of `value` in the SortedArray.
     *
     * @example
     * const sl = new SortedArray([1, 2, 2, 3, 3, 3, 4, 4, 4, 4]);
     * sl.count(3) // 3
     *
     * @param value - Value to count in SortedArray.
     * @returns Count.
     */
    count(value: T): number;
    /**
     * Return a shallow copy of the SortedArray.
     *
     * @returns A new SortedArray.
     */
    clone(): SortedArray<T>;
    /**
     * Remove and return value at `index` in SortedArray.
     *
     * If the SortedArray is empty or index is out of range, `undefined` is returned and the SortedArray is not modified.
     *
     * Negative indices count back from the last item.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.pop() // 'e'
     * sl.pop(2) // 'c'
     * sl // SortedArray ['a', 'b', 'd']
     *
     * @param index - Index of value (default -1).
     * @returns Value or `undefined`.
     */
    pop(index?: number): T | undefined;
    /**
     * Return first index of `value` in SortedArray, or -1 if `value` is not present.
     *
     * Index must be between `start` and `end` for the `value` to be considered present.
     * Negative indices count back from the last item.
     *
     * @example
     * const sl = new SortedArray('abcde');
     * sl.indexOf('d') // 3
     * sl.indexOf('z') // -1
     *
     * @param value - Value in SortedArray.
     * @param start - The array index at which to start the search. If omitted, defaults to 0.
     * @param end - The array index at which to end the search. If omitted, defaults to the end of the SortedArray.
     * @returns The index of the first occurrence of `value` in the SortedArray, or -1 if it is not present.
     */
    indexOf(value: T, start?: number, end?: number): number;
    /**
     * Return new SortedArray containing all values in both sequences.
     *
     * Values in `other` do not need to be in sorted order.
     *
     * @example
     * const sl1 = new SortedArray('bat');
     * const sl2 = new SortedArray('cat');
     * sl1.concat(sl2) // SortedArray ['a', 'a', 'b', 'c', 't', 't']
     *
     * @param other - Other iterable.
     * @returns New SortedArray.
     */
    concat(other: Iterable<T>): SortedArray<T>;
    /**
     * Return SortedArray as an ordinary Array.
     */
    toJSON(): T[];
    /**
     * Return a string representation of SortedArray.
     */
    toString(): string;
}
/**
 * Check invariants of SortedArray.
 *
 * This is a function instead of a method so that it can be tree-shaken.
 */
declare const checkSortedArray: <T>(self: SortedArray<T>) => void;

export { SortedArray, checkSortedArray };
